unit LicenseSpringUnit;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants,
  System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.OleServer, LicenseSpring,
  LicenseSpring_TLB,
  Vcl.StdCtrls, System.UITypes, ComObj, System.Generics.Collections,
  Winapi.ActiveX, Vcl.ExtCtrls;

type
  TLicenseSpringExampleForm = class(TForm)
    licenseManager: TLicenseManager;
    licenseSpringExtendedOptions: TExtendedOptions;
    apiKeyEdit: TEdit;
    sharedKeyEdit: TEdit;
    productCodeEdit: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    logMemo: TMemo;
    activateButton: TButton;
    deactivateAndRemoveCheckBox: TCheckBox;
    licenseEdit: TEdit;
    Label4: TLabel;
    LicenseSpringConfiguration1: TConfiguration;
    Customer1: TCustomer;
    activationTypeGroup: TRadioGroup;
    userEmail: TEdit;
    userPassword: TEdit;
    Label5: TLabel;
    Label6: TLabel;
    procedure activateButtonClick(Sender: TObject);
    procedure activationTypeGroupClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    procedure EnableAllButtons(enable: boolean);
    procedure CheckLicense(license: TLicense);
    procedure CheckLicenseLocal(license: TLicense);
    procedure PrintLicense(license: TLicense);
    procedure DeactivateOptional(license: TLicense);
    procedure ChoosedActivationType(index: integer);

  public
    { Public declarations }
  end;

var
  LicenseSpringExampleForm: TLicenseSpringExampleForm;

implementation

{$R *.dfm}

procedure TLicenseSpringExampleForm.activateButtonClick(Sender: TObject);
var
  licensekey: string;
  licenseID: TLicenseID;
  license: TLicense;
begin
  EnableAllButtons(false);
  logMemo.Clear;

  if (apiKeyEdit.Text = '') or (sharedKeyEdit.Text = '') or
    (productCodeEdit.Text = '') then
  begin
    MessageDlg('ApiKey, SharedKey and product code shouldn''t be empty!',
      mtInformation, [mbOk], 0);
    EnableAllButtons(true);
    Exit;
  end;

  LicenseSpringConfiguration1.apiKey := apiKeyEdit.Text;
  LicenseSpringConfiguration1.sharedKey := sharedKeyEdit.Text;
  LicenseSpringConfiguration1.productCode := productCodeEdit.Text;

  try
    licenseManager.LicenseFileName := 'testLicense.lic';

    license := licenseManager.CurrentLicense;

    If license = nil Then
    begin
      try
        // key based activation
        if activationTypeGroup.ItemIndex = 0 then
        begin
          licenseID := TLicenseID.FromKey(licenseEdit.Text);
          // 'assign some license key here to test or leave it empty to test trial key
          If licenseID.Key = '' Then
          begin
            licenseID := licenseManager.GetTrialLicense('');
            logMemo.Lines.Add
              (PWideChar(Format('%s %s', ['Got trial key from the server',
              licenseID.Key])));
          end;
        end
        // user based activation
        else if activationTypeGroup.ItemIndex = 1 then
        begin
          licenseID := TLicenseID.FromUser(userEmail.Text, userPassword.Text);
          // 'assign some license key here to test or leave it empty to test trial key
          If licenseID.IsEmpty Then
          begin
            licenseID := licenseManager.GetTrialLicense(Customer1, '');
            logMemo.Lines.Add
              (PWideChar(Format('%s %s', ['Got trial key from the server',
              licenseID.Key])));
          end;
        end;

        license := licenseManager.ActivateLicense(licenseID);
        logMemo.Lines.Add(PWideChar(Format('%s %s %s', ['License key',
          license.Key, ' activated successfully'])));
      finally
        FreeAndNil(licenseID);
      end;
    end
    else
    begin
      CheckLicenseLocal(license);
    end;
    CheckLicense(license);

    PrintLicense(license);
    DeactivateOptional(license);
  except
    on e: Exception do
      // MessageDlg(Format('0x%x', [e.ErrorCode]) + ': ' + e.Message, mtError, [mbOK], 0);
      MessageDlg(e.Message, mtError, [mbOk], 0);
  end;

  EnableAllButtons(true);
  FreeAndNil(license);
end;

procedure TLicenseSpringExampleForm.activationTypeGroupClick(Sender: TObject);
begin
  ChoosedActivationType(activationTypeGroup.ItemIndex);
end;

procedure TLicenseSpringExampleForm.CheckLicense(license: TLicense);
begin
  // 'it's recommended to perform online check from time to time
  logMemo.Lines.Add('Checking license online...');
  license.Check();
  // this call can throws errors like not enabled, expired, etc
  logMemo.Lines.Add('License successfully checked');
end;

procedure TLicenseSpringExampleForm.CheckLicenseLocal(license: TLicense);
begin
  logMemo.Lines.Add(PWideChar(Format('%s %s',
    ['License successfully loaded from file:',
    licenseManager.LicenseFilePath])));
  // 'it's recommended to perform a localCheck (offline check) on every startup
  // 'to be ensure that license file wasn't copied from another PC and license in a valid state
  logMemo.Lines.Add('Performing local check of the license...');
  license.LocalCheck;
  logMemo.Lines.Add('Local validation successful');
end;

procedure TLicenseSpringExampleForm.ChoosedActivationType(index: integer);
begin
  case index of
    - 1:
      begin
        licenseEdit.Enabled := false;
        userEmail.Enabled := false;
        userPassword.Enabled := false;
      end;
    0:
      begin
        licenseEdit.Enabled := true;
        userEmail.Enabled := false;
        userPassword.Enabled := false;
      end;
    1:
      begin
        licenseEdit.Enabled := false;
        userEmail.Enabled := true;
        userPassword.Enabled := true;
      end;

  end;
end;

procedure TLicenseSpringExampleForm.DeactivateOptional(license: TLicense);
begin
  if deactivateAndRemoveCheckBox.Checked then
  begin
    license.Deactivate(true);
    licenseManager.ClearLocalStorage;
    logMemo.Lines.Add('License was deactivated and removed!');
  end;
end;

procedure TLicenseSpringExampleForm.EnableAllButtons(enable: boolean);
begin
  activateButton.Enabled := enable;
  deactivateAndRemoveCheckBox.Enabled := enable;
  apiKeyEdit.Enabled := enable;
  sharedKeyEdit.Enabled := enable;
  productCodeEdit.Enabled := enable;
  if enable then
    ChoosedActivationType(activationTypeGroup.ItemIndex)
  else
    ChoosedActivationType(-1);
end;

procedure TLicenseSpringExampleForm.FormCreate(Sender: TObject);
begin
  ChoosedActivationType(activationTypeGroup.ItemIndex);
end;

procedure TLicenseSpringExampleForm.PrintLicense(license: TLicense);
var
  licenseStatus: TLicenseStatus;
  licFeatures: TLicenseFeaturesArray;
  customFields: TCustomFieldsArray;
  len, i: integer;
begin
  logMemo.Lines.Add('------------- License info -------------');
  logMemo.Lines.Add(PWideChar(Format('%s %s', ['Key: ', license.Key])));
  logMemo.Lines.Add(PWideChar(Format('%s %d', ['Type:', license.type_])));
  licenseStatus := license.Status;
  logMemo.Lines.Add(PWideChar(Format('%s %s', ['Status:',
    licenseStatus.ToString])));
  logMemo.Lines.Add(PWideChar(Format('%s %s', ['IsTrial: ',
    BoolToStr(license.IsTrial, true)])));
  logMemo.Lines.Add(PWideChar(Format('%s %s', ['IsOfflineActivated: ',
    BoolToStr(license.IsOfflineActivated, true)])));
  logMemo.Lines.Add(PWideChar(Format('%s %s', ['Validity Period:',
    FormatDateTime('dd-mm-yyyy', license.ValidityPeriod)])));
  logMemo.Lines.Add(PWideChar(Format('%s %s', ['Validity Period UTC:',
    FormatDateTime('dd-mm-yyyy', license.ValidityPeriodUTC)])));
  logMemo.Lines.Add(PWideChar(Format('%s %d', ['Grace Period: ', license.GracePeriod])));
  logMemo.Lines.Add(PWideChar(Format('%s %s', ['Validity With Grace Period:',
    FormatDateTime('dd-mm-yyyy', license.ValidityWIthGracePeriod)])));
  logMemo.Lines.Add(PWideChar(Format('%s %s', ['Validity With Grace Period UTC:',
    FormatDateTime('dd-mm-yyyy', license.ValidityWithGracePeriodUTC)])));
  logMemo.Lines.Add(PWideChar(Format('%s %s', ['Maintenance period:',
    FormatDateTime('dd-mm-yyyy', license.MaintenancePeriod)])));
  logMemo.Lines.Add(PWideChar(Format('%s %s', ['Maintenance period:',
    FormatDateTime('dd-mm-yyyy', license.MaintenancePeriodUTC)])));
  logMemo.Lines.Add(PWideChar(Format('%s %s', ['Maintenance period:',
    FormatDateTime('dd-mm-yyyy', license.LastCheckDate)])));
  logMemo.Lines.Add(PWideChar(Format('%s %d',
    ['Days passed since last online check:',
    license.DaysPassedSinceLastCheck])));

  licFeatures := license.Features;
  if licFeatures <> nil then
  begin
    logMemo.Lines.Add('License features:');
    len := length(licFeatures);
    for i := 0 to len - 1 do
    begin
      logMemo.Lines.Add(licFeatures[i].ToString);
    end;
  end;

  customFields := license.customFields;
  if customFields <> nil then
  begin
    logMemo.Lines.Add('License CustomFields:');
    len := length(customFields);
    for i := 0 to len - 1 do
    begin
      logMemo.Lines.Add(PWideChar(Format('%s: %s', [customFields[i].Name,
        customFields[i].Value])));
    end;
  end;
  FreeAndNil(licenseStatus);
  len := length(licFeatures);
  for i := 0 to len - 1 do
  begin
    FreeAndNil(licFeatures[i]);
  end;

  len := length(customFields);
  for i := 0 to len - 1 do
  begin
    FreeAndNil(customFields[i]);
  end;
end;

end.
